/**




   The static method parse() is a recursive descent parser for JSON strings.

   Here is a EBNF grammar for JSON strings.

 object -> '{' [ pair ( ',' pair)* ] '}'

   pair -> string ':' value

  array -> '[' [ value ( ',' value)* ] ']'

  value -> object   // begins with '{'
         | array    // begins with '['
         | true
         | false
         | null
         | string
         | number
*/

public class ParseJSON2AST
{
   public static Tree parse(String prefix) throws ParseException
   {
      Tokenizer tokens = new Tokenizer(prefix);

      Tree result = getObject(tokens);        // parse the token stream

      if ( tokens.hasToken() )  // there shouldn't be any more tokens
         throw new ParseException("unexpected input: "+ tokens.peekToken() + "\n" + tokens);

      return result;
   }//parse()

   private static Tree getObject(Tokenizer tokens) throws ParseException
   {
      Tree result = new Tree("Object");

      String token = null;

      if ( tokens.hasToken() )
      {
         token = tokens.nextToken();
         if ( ! token.equals("{") )
            throw new ParseException("getObject() expected '{' found '" + token + "'");
         //else
         //   result.addSubTree( new Tree("{") );
      }
      else
         throw new ParseException("unexpected end of input: " + "\n" + tokens);

      if ( tokens.hasToken() )
      {
         token = tokens.peekToken();  // one token look ahead
      }
      else
         throw new ParseException("unexpected end of input: " + "\n" + tokens);

      if ( ! token.equals("}") )
      {
         result.addSubTree( getPair(tokens) );

         if ( tokens.hasToken() )
         {
            token = tokens.peekToken();  // one token look ahead
         }
         else
            throw new ParseException("unexpected end of input: " + "\n" + tokens);

         while ( token.equals(",") )
         {
            tokens.nextToken();  // consume the comma
          //result.addSubTree( new Tree(tokens.nextToken()) ); // consume the comma
            result.addSubTree( getPair(tokens) );
            if ( tokens.hasToken() )
            {
               token = tokens.peekToken();  // one token look ahead
            }
            else
               throw new ParseException("unexpected end of input: " + "\n" + tokens);
         }
      }

      token = tokens.nextToken();
      if ( ! token.equals("}") )
         throw new ParseException("getObject() expected ',' or '}' found '" + token + "'");
    //else
    //   result.addSubTree( new Tree("}") );

      return result;
    }//getObject()


   private static Tree getPair(Tokenizer tokens) throws ParseException
   {
      Tree result = new Tree("Pair");

      if ( tokens.hasToken() )
      {
         result.addSubTree( new Tree(tokens.nextToken()) );  // consume the string
      }
      else
         throw new ParseException("unexpected end of input: " + "\n" + tokens);

      if ( tokens.hasToken() )
      {
         String token = tokens.nextToken();
         if ( ! token.equals(":") )
            throw new ParseException("getPair() expected ':' found '" + token + "'");
         //else
         //   result.addSubTree( new Tree(token) );
      }
      else
         throw new ParseException("unexpected end of input: " + "\n" + tokens);

      if ( tokens.hasToken() )
      {
         result.addSubTree( getValue(tokens) );
      }
      else
         throw new ParseException("unexpected end of input: " + "\n" + tokens);

      return result;
   }//getPair()


   private static Tree getArray(Tokenizer tokens) throws ParseException
   {
      Tree result = new Tree("Array");

      String token = null;

      if ( tokens.hasToken() )
      {
         token = tokens.nextToken();
         if ( ! token.equals("[") )
            throw new ParseException("getArray() expected '[' found '" + token + "'");
         //else
         //   result.addSubTree( new Tree("[") );
      }
      else
         throw new ParseException("unexpected end of input: " + "\n" + tokens);

      if ( tokens.hasToken() )
      {
         token = tokens.peekToken();  // one token look ahead
      }
      else
         throw new ParseException("unexpected end of input: " + "\n" + tokens);

      if ( ! token.equals("]") )
      {
         result.addSubTree( getValue(tokens) );

         if ( tokens.hasToken() )
         {
            token = tokens.peekToken();  // one token look ahead
         }
         else
            throw new ParseException("unexpected end of input: " + "\n" + tokens);

         while ( token.equals(",") )
         {
            tokens.nextToken();  // consume the comma
          //result.addSubTree( new Tree(tokens.nextToken()) ); // consume the comma
            result.addSubTree( getValue(tokens) );
            if ( tokens.hasToken() )
            {
               token = tokens.peekToken();  // one token look ahead
            }
            else
               throw new ParseException("unexpected end of input: " + "\n" + tokens);
         }
      }

      token = tokens.nextToken();
      if ( ! token.equals("]") )
         throw new ParseException("getArray() expected ',' or ']' found '" + token + "'");
      //else
      //   result.addSubTree( new Tree("]") );

      return result;
   }//getArray()


   private static Tree getValue(Tokenizer tokens) throws ParseException
   {
      Tree result = null;

      String token = null;

      if ( tokens.hasToken() )
      {
         token = tokens.peekToken();  // one token look ahead
      }
      else
         throw new ParseException("unexpected end of input: " + "\n" + tokens);

      if ( token.equals("{") )
      {
         result = getObject(tokens);
      }
      else if ( token.equals("[") )
      {
         result = getArray(tokens);
      }
      else if ( token.equals("true") )
      {
         result = new Tree(tokens.nextToken());    // consume the literal
      }
      else if ( token.equals("false") )
      {
         result = new Tree(tokens.nextToken());     // consume the literal
      }
      else if ( token.equals("null") )
      {
         result = new Tree(tokens.nextToken());     // consume the literal
      }
      else  // number or string
      {
         result = new Tree(tokens.nextToken());
      }
      return result;
   }//getValue()

}//ParseJSON2AST
