/*
   Silly Language S1.

   This parser does not "do" anything. It does not create a parse tree
   nor an abstract syntax tree. This parser is just a "recognizer" for
   the language. It will throw an exception if the string cannot be parsed.

   Notice that these productions are mutually recursive, but no production is recursive.

   S1 -> A '+' B
       | B '+' A

   A  -> 'a' '*' '(' S1 ')'
       | 'a'

   B  -> 'b' '*' '(' S1 ')'
       | 'b'

   We can use EBNF to rewrite this grammar.

   S1 -> A '+' B
       | B '+' A

   A  -> 'a' [ '*' '(' S1 ')' ]

   B  -> 'b' [ '*' '(' S1 ')' ]

   Some example strings are:
   "a*(a+b)+b"
   "b*(a*(b+a)+b)+a*(a+b*(b+a))"
*/

public class S1Parse
{
   public static void recognizeS1(String stringInS1) throws ParseException
   {
      Tokenizer tokens = new Tokenizer(stringInS1);

      getS1(tokens);
   }//parecognizeS1()


   public static void getS1(Tokenizer tokens) throws ParseException
   {
      String token = null;

      if ( tokens.hasToken() )
      {
         token = tokens.peekToken();  // one token look ahead
      }
      else
         throw new ParseException("unexpected end of input: " + "\n" + tokens);

      if ( token.equals("a") )            // decide which case we're in
      {
         getA(tokens);
         token = tokens.nextToken();
         if ( ! token.equals("+") )
            throw new ParseException("Can't parse " + token);
         getB(tokens);
      }
      else if ( token.equals("b") )
      {
         getB(tokens);
         token = tokens.nextToken();
         if ( ! token.equals("+") )
            throw new ParseException("Can't parse " + token);
         getA(tokens);
      }
      else
         throw new ParseException("Can't parse.");
   }//getS1()


   public static void getA(Tokenizer tokens) throws ParseException
   {
      String token = tokens.nextToken();
      if ( ! token.equals("a") )
         throw new ParseException("Can't parse " + token);
      if ( tokens.hasToken() )
      {
         token = tokens.peekToken();  // we "peek" ahead to help us
         if ( token.equals("*") )     // decide what to do next
         {
            tokens.nextToken();  // consume the *
            token = tokens.nextToken();
            if ( ! token.equals("(") )
               throw new ParseException("Can't parse " + token);
            getS1(tokens);
            token = tokens.nextToken();
            if ( ! token.equals(")") )
               throw new ParseException("Can't parse " + token);
         }
      }
   }//getA()


   public static void getB(Tokenizer tokens) throws ParseException
   {
      String token = tokens.nextToken();
      if ( ! token.equals("b") )
         throw new ParseException("Can't parse " + token);
      if ( tokens.hasToken() )
      {
         token = tokens.peekToken();  // we "peek" ahead to help us
         if ( token.equals("*") )     // decide what to do next
         {
            tokens.nextToken();  // consume the *
            token = tokens.nextToken();
            if ( ! token.equals("(") )
               throw new ParseException("Can't parse " + token);
            getS1(tokens);
            token = tokens.nextToken();
            if ( ! token.equals(")") )
               throw new ParseException("Can't parse " + token);
         }
      }
   }//getB()





   public static void main(String[] args)
   {
      String[] stringsInS1 = {
                               "a * ( a + b ) + b",
                               "b * ( a * ( b + a ) + b ) + a * ( a + b * ( b + a ) )",
                               "b * ( a * ( b + a ) + b ) + a * ( b + b * ( b + a ) )"
                             };

      for (int i = 0; i < stringsInS1.length; i++)
      {
         try
         {
            System.out.println( "Parsing " + stringsInS1[i] );
            recognizeS1( stringsInS1[i] );
            System.out.println( " ... Parsed" );
         }
         catch (ParseException e)
         {
            System.out.println(e);
          //e.printStackTrace();
         }
      }
   }//main()

}//S1Parse
