/*
   Silly Language S1.

   This parser creates a syntax tree for the expression.



   Notice that these productions are mutually recursive, but no production is recursive.

   S1 -> A '+' B
       | B '+' A

   A  -> 'a' '*' '(' S1 ')'
       | 'a'

   B  -> 'b' '*' '(' S1 ')'
       | 'b'

   We can use EBNF to rewrite this grammar.

   S1 -> A '+' B
       | B '+' A

   A  -> 'a' [ '*' '(' S1 ')' ]

   B  -> 'b' [ '*' '(' S1 ')' ]

   Some example strings are:
   "a*(a+b)+b"
   "b*(a*(b+a)+b)+a*(a+b*(b+a))"
*/

public class S1ParseTree
{
   public static Tree parseS1(String stringInS1) throws ParseException
   {
      Tokenizer tokens = new Tokenizer(stringInS1);

      return getS1(tokens);
   }//parecognizeS1()


   public static Tree getS1(Tokenizer tokens) throws ParseException
   {
      Tree result = new Tree( "S1" );

      String token = null;

      if ( tokens.hasToken() )
      {
         token = tokens.peekToken();  // one token look ahead
      }
      else
         throw new ParseException("unexpected end of input: " + "\n" + tokens);

      if ( token.equals("a") )            // decide which case we're in
      {
         result.addSubTree( getA(tokens) );
         token = tokens.nextToken();
         if ( !token.equals("+") )
         {
            throw new ParseException("Can't parse " + token);
         }
         else
         {
            result.addSubTree( new Tree("+") );
         }
         result.addSubTree( getB(tokens) );
      }
      else if ( token.equals("b") )
      {
         result.addSubTree( getB(tokens) );
         token = tokens.nextToken();
         if ( !token.equals("+") )
         {
            throw new ParseException("Can't parse " + token);
         }
         else
         {
            result.addSubTree( new Tree("+") );
         }
         result.addSubTree( getA(tokens) );
      }
      else
         throw new ParseException("Can't parse.");

      return result;
   }//getS1()


   public static Tree getA(Tokenizer tokens) throws ParseException
   {
      Tree result = new Tree( "A" );

      String token = tokens.nextToken();
      if ( !token.equals("a") )
      {
         throw new ParseException("Can't parse " + token);
      }
      else
      {
         result.addSubTree( new Tree("a") );
      }
      if ( tokens.hasToken() )
      {
         token = tokens.peekToken();  // we "peek" ahead to help us
         if ( token.equals("*") )     // decide what to do next
         {
            tokens.nextToken();  // consume the *
            result.addSubTree( new Tree("*") );
            token = tokens.nextToken();
            if ( !token.equals("(") )
            {
               throw new ParseException("Can't parse " + token);
            }
            else
            {
               result.addSubTree( new Tree("(") );
            }
            result.addSubTree( getS1(tokens) );
            token = tokens.nextToken();
            if ( !token.equals(")") )
            {
               throw new ParseException("Can't parse " + token);
            }
            else
            {
               result.addSubTree( new Tree(")") );
            }
         }
      }

      return result;
   }//getA()


   public static Tree getB(Tokenizer tokens) throws ParseException
   {
      Tree result = new Tree( "B" );

      String token = tokens.nextToken();
      if ( !token.equals("b") )
      {
         throw new ParseException("Can't parse " + token);
      }
      else
      {
         result.addSubTree( new Tree("b") );
      }
      if ( tokens.hasToken() )
      {
         token = tokens.peekToken();  // we "peek" ahead to help us
         if ( token.equals("*") )     // decide what to do next
         {
            tokens.nextToken();  // consume the *
            result.addSubTree( new Tree("*") );
            token = tokens.nextToken();
            if ( !token.equals("(") )
            {
               throw new ParseException("Can't parse " + token);
            }
            else
            {
               result.addSubTree( new Tree("(") );
            }
            result.addSubTree( getS1(tokens) );
            token = tokens.nextToken();
            if ( !token.equals(")") )
            {
               throw new ParseException("Can't parse " + token);
            }
            else
            {
               result.addSubTree( new Tree(")") );
            }
         }
      }

      return result;
   }//getB()





   public static void main(String[] args)
   {
      String[] stringsInS1 = {
                               "a * ( a + b ) + b",
                               "b * ( a * ( b + a ) + b ) + a * ( a + b * ( b + a ) )",
                               "b * ( a * ( b + a ) + b ) + a * ( b + b * ( b + a ) )"
                             };

      for (int i = 0; i < stringsInS1.length; i++)
      {
         try
         {
            Tree parseTree = parseS1( stringsInS1[i] );
            try  // Create dot and png files from the parse tree.
            {
               // Create the (empty) dot file.
               String baseName = String.format("S1_%02d__ParseTree", i);
               java.io.PrintWriter out = new java.io.PrintWriter(
                                         new java.io.File(baseName + ".dot") );
               // Put dot commands into the dot file
               out.println("//  " + parseTree + "\n");
               out.println( Tree2dot.tree2dot(parseTree) + "\n" );
               out.close();
               // Create a command line for running dot.exe.
               String[] cmd = {"C:\\graphviz-2.38\\release\\bin\\dot.exe",
                               "-Tpng",
                               baseName + ".dot",
                               "-o",
                               baseName + ".png"};
               // Execute the command line.
               java.lang.Runtime.getRuntime().exec(cmd);
            }
            catch (Exception e)
            {
               System.out.println( e );
            }
         }
         catch (ParseException e)
         {
            System.out.println(e);
          //e.printStackTrace();
         }
      }
   }//main()

}//S1ParseTree
