/**
   This program tests the Parse2ParseTree parser.

   When this program runs, it creates dot and
   png files for each of the parse trees.

   You can download the installer for dot.exe from this URL.
     https://graphviz.gitlab.io/_pages/Download/windows/graphviz-2.38.zip
*/

public class TestPostfix2
{
   public static void main(String[] args)
   {
      String postfix[] =
            // postfix notation       // parenthesized infix notation
          { "2 3 4 * +",              // 2 + 3 * 4
            "2 3 + 4 *",              // (2 + 3) * 4
            "2 3 - 4 +",              // 2 - 3 + 4
            "2 3 4 + -",              // 2 - (3 + 4)
            "2 3 ^ neg",              // -2^3
            "2 neg 3 ^",              // (-2)^3
            "5 neg 5 2 ^ 4 11 * 12 * - sqrt + 2 11 * /",
            "3 4 ^ 7 8 *",            // syntax error (at end)
            "3 4 ^ 7 * +",            // syntax error (at +)
            "3 4 ^ 7 8 @ +",          // syntax error (at @)
            "3 4 ^ 7 x * +",          // syntax error (at x)
            "3 4 ^ 7 8 * neg"         // syntax error (at end)
          };


      for (int i = 0; i < postfix.length; i++)
      {
         try  // Build the parse tree for the expression.
         {
            Tree parseTree = Parse2ParseTree.parse( postfix[i] );
            System.out.println( parseTree + "\n" );

            try  // Create dot and png files from the parse tree.
            {
               // Create the (empty) dot file.
               String baseName = String.format("Postfix_Expression_%02d_ParseTree", i);
               java.io.PrintWriter out = new java.io.PrintWriter(
                                         new java.io.File(baseName + ".dot") );
               // Put dot commands into the dot file
               out.println("//  " + parseTree + "\n");
               out.println( Tree2dot.tree2dot(parseTree) + "\n" );
               out.close();
               // Create a command line for running dot.exe.
               String[] cmd = {"C:\\graphviz-2.38\\release\\bin\\dot.exe",
                               "-Tpng",
                               baseName + ".dot",
                               "-o",
                               baseName + ".png"};
               // Execute the command line.
               java.lang.Runtime.getRuntime().exec(cmd);
            }
            catch (Exception e)
            {
               System.out.println(e);
            }
         }
         catch (Exception e)
         {
            System.out.println(e + "\n");
          //e.printStackTrace();
         }
      }
   }//main()

}//TestPostfix2
